
void format_demo (ostream & o) {
    o << "\n\n format_demo ...";
    o.precision (1);

    mcd0 const a   {+1};
    mcd1 const b   {+1, -2};
    mcd2 const c  {{+1, -2}, {-3,  +4}};
    mcd3 const d {{{+1, -2}, {-3,  +4}}, {{+5, -6}, { +7, +8}}};

    o << "\n\n view_i:";
    o << "\n a:    "; a.view_i (o);
    o << "\n b:   "; b.view_i (o);
    o << "\n c:  "; c.view_i (o);
    o << "\n d: "; d.view_i (o);

    o << "\n\n view_j:";
    o << "\n a: "; a.view_j (o);
    o << "\n b: "; b.view_j (o);
    o << "\n c: "; c.view_j (o);
    o << "\n d: "; d.view_j (o);

    o << "\n\n view_k:";
    o << "\n a:    "; a.view_k (o);
    o << "\n b:   "; b.view_k (o);
    o << "\n c:  "; c.view_k (o);
    o << "\n d: "; d.view_k (o);
}

void unit_demo (ostream & o) {
    o << "\n\n unit_demo ...";
    o.precision (1);

    o << "\n";
    o << "\n  zero0:          " << mcd0::zero;
    o << "\n  zero1:         "  << mcd1::zero;
    o << "\n  zero2:        "   << mcd2::zero;
    o << "\n  zero3:       "    << mcd3::zero;
    o << "\n  zero4:      "     << mcd4::zero;

    o << "\n";
    o << "\n  real0:          " << mcd0::real_unit;
    o << "\n  real1:         "  << mcd1::real_unit;
    o << "\n  real2:        "   << mcd2::real_unit;
    o << "\n  real3:       "    << mcd3::real_unit;
    o << "\n  real4:      "     << mcd4::real_unit;

    o << "\n";
    o << "\n  imag 0:         does not exist ";
    o << "\n  imag 1:        " << mcd1::imag_unit;
    o << "\n  imag 2:       "  << mcd2::imag_unit;
    o << "\n  imag 3:      "   << mcd3::imag_unit;
    o << "\n  imag 4:     "    << mcd4::imag_unit;

    o << "\n";
    o << "\n (imag 1)^2:    "   << mcd1::imag_unit * mcd1::imag_unit;
    o << "\n (imag 2)^2:   "    << mcd2::imag_unit * mcd2::imag_unit;
    o << "\n (imag 3)^2:  "     << mcd3::imag_unit * mcd3::imag_unit;
    o << "\n (imag 4)^2: "      << mcd4::imag_unit * mcd4::imag_unit;
}

template <typename mcd_a, typename mcd_b, typename mcd_c>
void arith_demo (ostream & o) {
    o << "\n\n arith_demo ...";
    o.precision (5);

    auto const a (mcd_a::random());
    auto const b (mcd_b::random());
    auto const c (mcd_c::random());

    o << "\n";
    o << "\n a:             " << a;
    o << "\n b:             " << b;
    o << "\n c:             " << c;

    o << "\n\n the numbers in each of the following groups ";
    o << "\n should be equal, aside from zero-padding: ";

    o << "\n";
    o << "\n a + b:         " << a + b;
    o << "\n b + a:         " << b + a;
    o << "\n a - (- b):     " << a - (- b);
    o << "\n";
    o << "\n a - b:         " << a - b;
    o << "\n a + (- b):     " << a + (- b);
    o << "\n - (b - a):     " << - (b - a);
    o << "\n";
    o << "\n a:             " << a;
    o << "\n (a + b) - b:   " << (a + b) - b;
    o << "\n (a - b) + b:   " << (a - b) + b;
    o << "\n";
    o << "\n (a + b) + c:   " << (a + b) + c;
    o << "\n a + (b + c):   " << a + (b + c);
    o << "\n";
    o << "\n a * b:         " << a * b;
    o << "\n b * a:         " << b * a;
    o << "\n";
    o << "\n a:             " << a;
    o << "\n (a * b) / b:   " << (a * b) / b;
    o << "\n (a / b) * b:   " << (a / b) * b;
    o << "\n";
    o << "\n b:             " << b;
    o << "\n (b * a) / a:   " << (b * a) / a;
    o << "\n (b / a) * a:   " << (b / a) * a;
    o << "\n";
    o << "\n a * (b + c):   " << a * (b + c);
    o << "\n a * b + a * c: " << a * b + a * c;
    o << "\n";
    o << "\n a * (b - c):   " << a * (b - c);
    o << "\n a * b - a * c: " << a * b - a * c;
    o << "\n";
    o << "\n (a + b) * c:   " << (a + b) * c;
    o << "\n a * c + b * c: " << a * c + b * c;
    o << "\n";
    o << "\n (a - b) * c:   " << (a - b) * c;
    o << "\n a * c - b * c: " << a * c - b * c;
    o << "\n";
    o << "\n (a + b) / c:   " << (a + b) / c;
    o << "\n a / c + b / c: " << a / c + b / c;
    o << "\n";
    o << "\n (a - b) / c:   " << (a - b) / c;
    o << "\n a / c - b / c: " << a / c - b / c;
}

void arith_demo_rigorous (ostream & o) {
    // this function exists mainly to see if
    // all those templates really will compile

    // none zero
        // all different
            arith_demo <mcd1, mcd2, mcd3> (o);
            arith_demo <mcd1, mcd3, mcd2> (o);
            arith_demo <mcd2, mcd3, mcd1> (o);
            arith_demo <mcd2, mcd1, mcd3> (o);
            arith_demo <mcd3, mcd1, mcd2> (o);
            arith_demo <mcd3, mcd2, mcd1> (o);
        // two the same
            arith_demo <mcd2, mcd2, mcd1> (o);
            arith_demo <mcd2, mcd2, mcd3> (o);
            arith_demo <mcd2, mcd1, mcd2> (o);
            arith_demo <mcd2, mcd3, mcd2> (o);
            arith_demo <mcd1, mcd2, mcd2> (o);
            arith_demo <mcd3, mcd2, mcd2> (o);
        // all the same
            arith_demo <mcd2, mcd2, mcd2> (o);

    // some zero
        // all different
            arith_demo <mcd0, mcd1, mcd2> (o);
            arith_demo <mcd0, mcd2, mcd1> (o);
            arith_demo <mcd1, mcd2, mcd0> (o);
            arith_demo <mcd1, mcd0, mcd2> (o);
            arith_demo <mcd2, mcd0, mcd1> (o);
            arith_demo <mcd2, mcd1, mcd0> (o);
        // two the same
            arith_demo <mcd0, mcd0, mcd2> (o);
            arith_demo <mcd2, mcd2, mcd0> (o);
            arith_demo <mcd0, mcd2, mcd0> (o);
            arith_demo <mcd2, mcd0, mcd2> (o);
            arith_demo <mcd2, mcd0, mcd0> (o);
            arith_demo <mcd0, mcd2, mcd2> (o);
        // all the same
            arith_demo <mcd0, mcd0, mcd0> (o);


}