
void transcend_demo_1 (ostream & o) {
    o << "\n\n transcend_demo_1 ...";

    auto const a (mcd2::random());
    auto const b (mcd3::random());

    static auto const imag (mcd2::imag_unit);

    o << "\n";
    o << "\n a: " << a;
    o << "\n b: " << b;

    auto const sin_a (sin (a));
    auto const sin_b (sin (b));
    auto const cos_a (cos (a));
    auto const cos_b (cos (b));

    auto const sinh_a (sinh (a));
    auto const sinh_b (sinh (b));
    auto const cosh_a (cosh (a));
    auto const cosh_b (cosh (b));

    {
        o << "\n\n these should be equal: ";
        o << "\n";
        o << "\n " << mcd3::real_unit;
        o << "\n " << cos_a * cos_a + sin_a * sin_a;
        o << "\n " << cosh_a * cosh_a - sinh_a * sinh_a;
    } {
        o << "\n\n these pairs should be equal -- circular functions: ";
        o << "\n";
        o << "\n " << sin (a + b);
        o << "\n " << sin_a * cos_b + cos_a * sin_b;
        o << "\n";
        o << "\n " << cos (a + b);
        o << "\n " << cos_a * cos_b - sin_a * sin_b;
        o << "\n";
        o << "\n " << sin_a * sin_b * 2.0;
        o << "\n " << cos (a - b) - cos (a + b);
        o << "\n";
        o << "\n " << cos_a * cos_b * 2.0;
        o << "\n " << cos (a - b) + cos (a + b);
        o << "\n";
        o << "\n " << sin_a * cos_b * 2.0;
        o << "\n " << sin (a - b) + sin (a + b);
    } {
        o << "\n\n these pairs should be equal -- hyperbolic functions: ";
        o << "\n";
        o << "\n " << sinh (a + b);
        o << "\n " << sinh_a * cosh_b + cosh_a * sinh_b;
        o << "\n";
        o << "\n " << cosh (a + b);
        o << "\n " << cosh_a * cosh_b + sinh_a * sinh_b;
        o << "\n";
        o << "\n " << sinh_a * sinh_b * 2.0;
        o << "\n " << cosh (a + b) - cosh (a - b);
        o << "\n";
        o << "\n " << cosh_a * cosh_b * 2.0;
        o << "\n " << cosh (a + b) + cosh (a - b);
        o << "\n";
        o << "\n " << sinh_a * cosh_b * 2.0;
        o << "\n " << sinh (a + b) + sinh (a - b);
    } {
        o << "\n\n these pairs should be equal -- circular-hyperbolic conversion: ";
        o << "\n";
        o << "\n " << sin_a;
        o << "\n " << -imag * sinh (imag * a);
        o << "\n";
        o << "\n " << cos_a;
        o << "\n " << cosh (imag * a);
        o << "\n";
        o << "\n " << sinh_a;
        o << "\n " << -imag * sin (imag * a);
        o << "\n";
        o << "\n " << cosh_a;
        o << "\n " << cos (imag * a);
    } {
        o << "\n\n these pairs should be equal -- exponential: ";
        o << "\n";
        o << "\n " << sin_a;
        o << "\n " << (exp (+a * imag) - exp (-a * imag)) * imag  / -2;
        o << "\n";
        o << "\n " << cos_a;
        o << "\n " << (exp (+a * imag) + exp (-a * imag)) / +2;
        o << "\n";
        o << "\n " << sinh_a;
        o << "\n " << (exp (+a) - exp (-a)) / +2;
        o << "\n";
        o << "\n " << cosh_a;
        o << "\n " << (exp (+a) + exp (-a)) / +2;
    }
}

template <int order>
typename std::enable_if <(order >= 1), void>::type
transcend_demo_2 (ostream & o) {
    o << "\n\n transcend_demo_2 ...";

    using mcdn = multicomplex<double,order>;

    auto const a (mcdn::random());
    auto const r (a.real);
    auto const i (a.imag);

    auto const sin_a  (sin  (a));
    auto const cos_a  (cos  (a));
    auto const sin_r  (sin  (r));
    auto const cos_r  (cos  (r));
    auto const sin_i  (sin  (i));
    auto const cos_i  (cos  (i));

    auto const sinh_a (sinh (a));
    auto const cosh_a (cosh (a));
    auto const sinh_r (sinh (r));
    auto const sinh_i (sinh (i));

    o << "\n";
    o << "\n a: " << a;

    o << "\n\n these pairs should be equal ";
    o << "\n";
    o << "\n " << sin_a.fold();
    o << "\n " << sin_r * sin_r + sinh_i * sinh_i;
    o << "\n";
    o << "\n " << cos_a.fold();
    o << "\n " << cos_r * cos_r + sinh_i * sinh_i;

    o << "\n";
    o << "\n " << sinh_a.fold();
    o << "\n " << sinh_r * sinh_r + sin_i * sin_i;
    o << "\n";
    o << "\n " << cosh_a.fold();
    o << "\n " << sinh_r * sinh_r + cos_i * cos_i;
}

void transcend_demo_3 (ostream & o) {
    o << "\n\n transcend_demo_3 ...";
    o << "\n maclaurin series approximation ";

    using mcdn = mcd3;

    auto const op (mcdn::random());
    auto const op2 (op * op);
    o << "\n\n operand: ";
    o << "\n " << op;

    {
        o << "\n\n figure sin ...";
        auto sum (mcdn::zero);
        auto numer (op);
        int denom {1};
        for (int n {1}; n < 15; n += 2) {
            sum = sum + numer / denom;
            o << "\n " << sum;
            numer = numer * op2;
            denom *= - (n + 1) * (n + 2);
        }
        o << "\n should equal: ";
        o << "\n " << sin (op);
    } {
        o << "\n\n figure cos ...";
        auto sum (mcdn::zero);
        auto numer (mcdn::real_unit);
        int denom {1};
        for (int n {0}; n < 14; n += 2) {
            sum = sum + numer / denom;
            o << "\n " << sum;
            numer = numer * op2;
            denom *= - (n + 1) * (n + 2);
        }
        o << "\n should equal: ";
        o << "\n " << cos (op);
    } {
        o << "\n\n figure sinh ...";
        auto sum (mcdn::zero);
        auto numer (op);
        int denom {1};
        for (int n {1}; n < 15; n += 2) {
            sum = sum + numer / denom;
            o << "\n " << sum;
            numer = numer * op2;
            denom *= + (n + 1) * (n + 2);
        }
        o << "\n should equal: ";
        o << "\n " << sinh (op);
    } {
        o << "\n\n figure cosh ...";
        auto sum (mcdn::zero);
        auto numer (mcdn::real_unit);
        int denom {1};
        for (int n {0}; n < 14; n += 2) {
            sum = sum + numer / denom;
            o << "\n " << sum;
            numer = numer * op2;
            denom *= + (n + 1) * (n + 2);
        }
        o << "\n should equal: ";
        o << "\n " << cosh (op);
    } {
        o << "\n\n figure exp ...";
        auto sum (mcdn::zero);
        auto numer (mcdn::real_unit);
        int denom {1};
        for (int n {0}; n < 13; n += 1) {
            sum = sum + numer / denom;
            o << "\n " << sum;
            numer = numer * op;
            denom *= + (n + 1);
        }
        o << "\n should equal: ";
        o << "\n " << exp (op);
    } {
        o << "\n\n pade approximant to exp ... ";
        auto const op3 (op2 * op);
        auto const numer (+ op3 + op2 * 12 + op * 60 + mcd0 {120});
        auto const denom (- op3 + op2 * 12 - op * 60 + mcd0 {120});
        cout << "\n " << numer / denom;
    }
}

