
void layout_demo (ostream & o) {
    o << "\n layout_demo ...";
    o.precision (2);

    multicomplex<double,3> const a {{{0.10,0.11},{0.12,0.13}},{{0.14,0.15},{0.16,0.17}}};
    multicomplex<double,3> const b {{{0.20,0.21},{0.22,0.23}},{{0.24,0.25},{0.26,0.27}}};
    multicomplex<double,3> const c {{{0.30,0.31},{0.32,0.33}},{{0.34,0.35},{0.36,0.37}}};
    multicomplex<double,3> const d {{{0.40,0.41},{0.42,0.43}},{{0.44,0.45},{0.46,0.47}}};

    using mcd5  = multicomplex<double,5>;
    using mcd32 = multicomplex<multicomplex<double,3>,2>;
    using mcd23 = multicomplex<multicomplex<double,2>,3>;

    mcd5  const x {{a,b},{c,d}};
    mcd32 const y {{a,b},{c,d}};
    mcd23 const z {
        {{a.real,a.imag},{b.real,b.imag}},
        {{c.real,c.imag},{d.real,d.imag}}
    };

    o << "\n\n x: " << x;
    o << "\n\n y: " << y;
    o << "\n\n z: " << z;

    o << "\n";
    if (std::is_trivially_copyable<mcd5 >()) cout << "\n mcd5  is trivially copyable";
    if (std::is_trivially_copyable<mcd32>()) cout << "\n mcd32 is trivially copyable";
    if (std::is_trivially_copyable<mcd23>()) cout << "\n mcd23 is trivially copyable";

    o << "\n";
    if (std::is_trivial<mcd5 >()) cout << "\n mcd5  is trivial";
    if (std::is_trivial<mcd32>()) cout << "\n mcd32 is trivial";
    if (std::is_trivial<mcd23>()) cout << "\n mcd23 is trivial";

    o << "\n";
    if (std::is_standard_layout<mcd5 >()) cout << "\n mcd5  is standard layout";
    if (std::is_standard_layout<mcd32>()) cout << "\n mcd32 is standard layout";
    if (std::is_standard_layout<mcd23>()) cout << "\n mcd23 is standard layout";

    o << "\n";
    if (std::is_pod<mcd5 >()) cout << "\n mcd5  is plain old data";
    if (std::is_pod<mcd32>()) cout << "\n mcd32 is plain old data";
    if (std::is_pod<mcd23>()) cout << "\n mcd23 is plain old data";

    o << "\n\n offsets, as measured in sizeof (char): ";

    o << "\n";
    o << "\n x.real:         " << (long int) (&x.real)         - (long int) (&x);
    o << "\n y.real:         " << (long int) (&y.real)         - (long int) (&y);
    o << "\n z.real:         " << (long int) (&z.real)         - (long int) (&z);
    o << "\n x.at_con<4>(0): " << (long int) (&x.at_con<4>(0)) - (long int) (&x);
    o << "\n y.at_con<1>(0): " << (long int) (&y.at_con<1>(0)) - (long int) (&y);
    o << "\n z.at_con<2>(0): " << (long int) (&z.at_con<2>(0)) - (long int) (&z);

    o << "\n";
    o << "\n x.real.imag:      " << (long int) (&x.real.imag)      - (long int) (&x);
    o << "\n y.real.imag:      " << (long int) (&y.real.imag)      - (long int) (&y);
    o << "\n y.real.imag.sole: " << (long int) (&y.real.imag.sole) - (long int) (&y);
    o << "\n z.real.imag:      " << (long int) (&z.real.imag)      - (long int) (&z);
    o << "\n x.at_con<3>(1):   " << (long int) (&x.at_con<3>(1))   - (long int) (&x);
    o << "\n y.at_con<0>(1):   " << (long int) (&y.at_con<0>(1))   - (long int) (&y);
    o << "\n z.at_con<1>(1):   " << (long int) (&z.at_con<1>(1))   - (long int) (&z);

    o << "\n";
    o << "\n x.real.imag.real:                 " << (long int) (&x.real.imag.real)                 - (long int) (&x);
    o << "\n y.real.imag.sole.real:            " << (long int) (&y.real.imag.sole.real)            - (long int) (&y);
    o << "\n z.real.imag.real:                 " << (long int) (&z.real.imag.real)                 - (long int) (&z);
    o << "\n z.real.imag.real.sole:            " << (long int) (&z.real.imag.real.sole)            - (long int) (&z);
    o << "\n x.at_con<2>(2):                   " << (long int) (&x.at_con<2>(2))                   - (long int) (&x);
    o << "\n y.at_con<0>(1).sole.at_con<2>(0): " << (long int) (&y.at_con<0>(1).sole.at_con<2>(0)) - (long int) (&y);
    o << "\n z.at_con<0>(2):                   " << (long int) (&z.at_con<0>(2))                   - (long int) (&z);

    o << "\n";
    o << "\n x.real.imag.real.imag:            " << (long int) (&x.real.imag.real.imag)            - (long int) (&x);
    o << "\n y.real.imag.sole.real.imag:       " << (long int) (&y.real.imag.sole.real.imag)       - (long int) (&y);
    o << "\n z.real.imag.real.sole.imag:       " << (long int) (&z.real.imag.real.sole.imag)       - (long int) (&z);
    o << "\n x.at_con<1>(5):                   " << (long int) (&x.at_con<1>(5))                   - (long int) (&x);
    o << "\n y.at_con<0>(1).sole.at_con<1>(1): " << (long int) (&y.at_con<0>(1).sole.at_con<1>(1)) - (long int) (&y);
    o << "\n z.at_con<0>(2).sole.at_con<1>(1): " << (long int) (&z.at_con<0>(2).sole.at_con<1>(1)) - (long int) (&z);

    o << "\n";
    o << "\n x.real.imag.real.imag.imag:        " << (long int) (&x.real.imag.real.imag.imag)        - (long int) (&x);
    o << "\n y.real.imag.sole.real.imag.imag:   " << (long int) (&y.real.imag.sole.real.imag.imag)   - (long int) (&y);
    o << "\n z.real.imag.real.sole.imag.imag:   " << (long int) (&z.real.imag.real.sole.imag.imag)   - (long int) (&z);
    o << "\n x.at_con<0>(11):                   " << (long int) (&x.at_con<0>(11))                   - (long int) (&x);
    o << "\n y.at_con<0>( 1).sole.at_con<0>(3): " << (long int) (&y.at_con<0>( 1).sole.at_con<0>(3)) - (long int) (&y);
    o << "\n z.at_con<0>( 2).sole.at_con<0>(3): " << (long int) (&z.at_con<0>( 2).sole.at_con<0>(3)) - (long int) (&z);
}