
template <typename elem>
class multicomplex<elem, 0> {
    void view_j (ostream &, int const) const;

    struct rand_double {
        rand_double () :
            r (bind(
                std::uniform_real_distribution<>(-1.0,+1.0),
                std::default_random_engine()
            )) { }
        double operator()() const { return r(); }
    private:
        std::function<double()> const r;
    };
    static rand_double get_random;
public:
    using elem_type = elem;
    static constexpr int order_value {0};

    elem sole; // this is the only data

/// constructors and related items

    multicomplex (
        elem const & g
    //  You might prefer this default:
    //  elem const & g = elem {}
    ) : sole {g} { }

    multicomplex () = delete;
    multicomplex (multicomplex const &) = default;
    multicomplex (multicomplex &&) = default;
    multicomplex & operator= (multicomplex const &) = default;
    multicomplex & operator= (multicomplex &&) = default;
    ~multicomplex () = default;

    static multicomplex const zero;
    static multicomplex const real_unit;
//  static multicomplex const imag_unit; // INTENTIONALLY OMITTED

    // WDTCRT means "why does the compiler requrire this?"
    template <typename WDTCRT = void>
    static typename std::enable_if <
        std::is_same<elem,double>::value
        && std::is_void<WDTCRT>::value
        , multicomplex>::type
    random ()
        { return get_random (); }

/// unary operators

    multicomplex operator+ () const
        { return multicomplex {+ sole}; }
    multicomplex operator~ () const
        { return multicomplex {+ sole}; }
    multicomplex operator- () const
        { return multicomplex {- sole}; }

/// arithmetic

    template <int o_order>
    multicomplex<elem,o_order> operator+ (multicomplex<elem,o_order> const & o) const
        { return multicomplex<elem,o_order> {*this + o.real, + o.imag}; }

    multicomplex<elem,0> operator+ (multicomplex<elem,0> const & o) const
        { return multicomplex {sole + o.sole}; }

    template <int o_order>
    multicomplex<elem,o_order> operator- (multicomplex<elem,o_order> const & o) const
        { return multicomplex<elem,o_order> {*this - o.real, - o.imag}; }

    multicomplex<elem,0> operator- (multicomplex<elem,0> const & o) const
        { return multicomplex {sole - o.sole}; }

    template <int o_order>
    multicomplex<elem,o_order> operator* (multicomplex<elem,o_order> const & o) const
        { return multicomplex<elem,o_order> {*this * o.real, *this * o.imag}; }

    multicomplex<elem,0> operator* (multicomplex<elem,0> const & o) const
        { return multicomplex {sole * o.sole}; }

    template <int o_order>
    multicomplex<elem,o_order> operator/ (multicomplex<elem,o_order> const & o) const
    {
        multicomplex<elem,o_order-1> const denom {o.fold()};
        multicomplex<elem,o_order-1> real {o.real / denom * sole};
        multicomplex<elem,o_order-1> imag {o.imag / denom * sole};
        return multicomplex<elem,o_order> {+real, -imag};
    }

    multicomplex<elem,0> operator/ (multicomplex<elem,0> const & o) const
        { return multicomplex {sole / o.sole}; }

    multicomplex operator* (elem const & o) const
        { return multicomplex {sole * o}; }
    multicomplex operator/ (elem const & o) const
        { return multicomplex {sole / o}; }

    multicomplex fold () const
        { return sole * sole; }

/// selection by position

private:
    template <int x_order>
    static typename
        std::enable_if <(order_value == x_order), multicomplex<elem,x_order> const &>::type
    priv_at_con (multicomplex<elem,0> const & source, int const n) {
        return source;
    }

    template <int x_order>
    static typename
        std::enable_if <(order_value == x_order), multicomplex<elem,x_order> &>::type
    priv_at_var (multicomplex<elem,0> & source, int const n) {
        return source;
    }

    // INTENTIONALLY OMITTED:
    //
    // template <int x_order>
    // static typename
    //     std::enable_if <(order != x_order), multicomplex<elem,x_order> const &>::type
    // priv_at_con (multicomplex<elem,0> const & source, int const n);
    //
    // template <int x_order>
    // static typename
    //     std::enable_if <(order != x_order), multicomplex<elem,x_order> &>::type
    // priv_at_var (multicomplex<elem,0> & source, int const n);
public:
    template <int x_order>
    multicomplex<elem,x_order> const & at_con (int const n) const {
        static_assert (x_order >= 0, "x_order < 0");
        static_assert (x_order <= order_value, "x_order > order");
        static int const full {1 << (order_value - x_order)};
        if (n < 0) throw fail {"multicomplex::at_con ... n < 0"};
        if (n >= full) throw fail {"multicomplex::at_con ... n >= full"};
        return priv_at_con<x_order> (*this, n);
    }

    template <int x_order>
    multicomplex<elem,x_order> & at_var (int const n) {
        static_assert (x_order >= 0, "x_order < 0");
        static_assert (x_order <= order_value, "x_order > order");
        static int const full {1 << (order_value - x_order)};
        if (n < 0) throw fail {"multicomplex::at_var ... n < 0"};
        if (n >= full) throw fail {"multicomplex::at_var ... n >= full"};
        return priv_at_var<x_order> (*this, n);
    }

/// miscellaneous

    elem norm () const
        { return sole * sole; }

    void view_i (ostream &) const;
    void view_j (ostream &) const;
    void view_k (ostream &) const;

    template<typename elem2, int order2>
    friend class multicomplex;
};

template <typename elem>
ostream & operator<< (ostream & o, multicomplex<elem, 0> const & mc) {
    mc.view_k (o); return o;
}

/////////////////////////////////////////

template <typename elem>
typename multicomplex<elem, 0>::rand_double multicomplex<elem, 0>::get_random;

template <typename elem, int order>
multicomplex<elem,order> const multicomplex<elem,order>::zero {
    multicomplex<elem,order-1>::zero,
    multicomplex<elem,order-1>::zero
};

template <typename elem>
multicomplex<elem,0> const multicomplex<elem,0>::zero {0};

template <typename elem, int order>
multicomplex<elem,order> const multicomplex<elem,order>::real_unit {
    multicomplex<elem,order-1>::real_unit,
    multicomplex<elem,order-1>::zero
};

template <typename elem>
multicomplex<elem,0> const multicomplex<elem,0>::real_unit {1};

template <typename elem, int order>
multicomplex<elem,order> const multicomplex<elem,order>::imag_unit {
    multicomplex<elem,order-1>::zero,
    multicomplex<elem,order-1>::real_unit
};

/////////////////////////////////////////

template <typename old_elem, typename new_elem, int order>
typename std::enable_if <(order > 0), multicomplex<new_elem,order>>::type
convert_elem (multicomplex<old_elem,order> const & o) {
    return multicomplex<new_elem,order> {
        convert_elem<old_elem, new_elem, order-1> (o.real),
        convert_elem<old_elem, new_elem, order-1> (o.imag)
    };
}

template <typename old_elem, typename new_elem, int order>
typename std::enable_if <(order == 0), multicomplex<new_elem,order>>::type
convert_elem (multicomplex<old_elem,order> const & o) {
    return multicomplex<new_elem,order> {
        new_elem (o.sole)
    };
}


